#ifndef CONTIGUOUS_3D_VECTOR_H_
#define CONTIGUOUS_3D_VECTOR_H_

#include <cstddef>

#define DEFAULT_OUTER_SIZE  3
#define DEFAULT_MIDDLE_SIZE 3
#define DEFAULT_INNER_SIZE  3

template <class T>
class Contiguous3Vector {
  
private:
  T*** m_array_outer;
  std::size_t m_size_outer;
  std::size_t m_size_middle;
  std::size_t m_size_inner;
	
public:
  Contiguous3Vector() :
	Contiguous3Vector(DEFAULT_OUTER_SIZE, DEFAULT_MIDDLE_SIZE, DEFAULT_INNER_SIZE) {}
  Contiguous3Vector(std::size_t _size_outer, std::size_t _size_middle, std::size_t _size_inner) {
	m_size_outer = _size_outer;
	m_size_middle = _size_middle;
	m_size_inner = _size_inner;
	m_array_outer = new T** [_size_outer];
	m_array_outer[0] = new T* [_size_outer * _size_middle];
	m_array_outer[0][0] = new T[_size_outer * _size_middle * _size_inner];

    for (std::size_t i = 0; i < _size_outer; i++) {
	  if (i > 0) {
		m_array_outer[i] = m_array_outer[i-1] + _size_middle;
		m_array_outer[i][0] = m_array_outer[i-1][0] + (_size_middle * _size_inner);
	  }      
	  for (std::size_t j = 1; j < _size_middle; j++) {
		m_array_outer[i][j] = m_array_outer[i][j-1] + _size_inner;
	  }
    }
  }

  /***************************** PROXY MIDDLE ****************************************/
  class ProxyMiddle {                                                                //
  public:                                                                           //
	ProxyMiddle(T **_array_middle, std::size_t _size_middle, std::size_t _size_inner) { //
	  m_array_middle = _array_middle;                                               //
	  m_size_middle = _size_middle;                                                 //
	  m_size_inner = _size_inner;                                                   //
	}                                                                               //
                                                                                    //
	/*************************** PROXY INNER **************************/            //
	class ProxyInner {                                                //            // 
	public:                                                           //            //
	  ProxyInner(T* _array_inner, std::size_t _size_inner) {          //            //
		m_array_inner = _array_inner;                                 //            //
		m_size_inner = _size_inner;                                   //            //
	  }                                                               //            //
	  // get                                                          //            //
	  const T & operator[](std::size_t index) const {                 //            //
		return m_array_inner[index];                                  //            //
	  }                                                               //            //
	  // set                                                          //            //
	  T & operator [](std::size_t index) {                            //            //
		return m_array_inner[index];                                  //            //
	  }                                                               //            //
	  std::size_t size() {                                            //            //
		return m_size_inner;                                          //            //
	  }                                                               //            //
	protected:                                                        //            //
	  T* m_array_inner;                                               //            //
	  std::size_t m_size_inner;                                       //            //
	};                                                                //            //
	/******************************************************************/            //
                                                                                    //
	ProxyInner operator[](std::size_t index) {                                      //
	  return ProxyInner(m_array_middle[index], m_size_inner);                       //
	}                                                                               //
	std::size_t size() {                                                            //
	  return m_size_middle;                                                         //
	}                                                                               //
  protected:                                                                        //
	T **m_array_middle;                                                             //
	std::size_t m_size_middle;                                                      //
	std::size_t m_size_inner;                                                       //
  };                                                                                //
  /**********************************************************************************/

  ProxyMiddle operator[](std::size_t index) {
	return ProxyMiddle(m_array_outer[index], m_size_middle, m_size_inner);
  }

  std::size_t size() {
	return m_size_outer;
  }

  T * data() {
	return &(m_array_outer[0][0][0]);
  }

  ~Contiguous3Vector() {
	delete [] m_array_outer[0];
	delete [] m_array_outer;
  }
};


#endif /* CONTIGUOUS_3D_VECTOR_H_ */
